const system = require("../../../system");
const moment = require('moment');
/**
 * 广电相关接口（RT）
 * laolan
 * 2020.11.11
 */
class RtService {
  constructor() {
    this.needsolutionDao = system.getObject("db.dbneed.needsolutionDao");
    this.needinfoDao = system.getObject("db.dbneed.needinfoDao");
    this.orderinfoDao = system.getObject("db.dbcorder.orderinfoDao");
    this.db = system.getObject("db.common.connection").getCon()
    this.rtStatusReference = {
      507: "完成账户注册",
      508: "服务商完成提交资料到⼯信部",
      509: "⼯商部已受理",
      510: "⼯商部不予受理",
      511: "⼯商部通过",
      512: "⼯商部未通过"
    };
    //网文状态2020-11-11 baidu
    this.rtSolutionStatus = {
      "SOLUTION_SUBMIT": "提交服务单",
      "MATERIAL_UNCONFIRM": "服务商递交⽂件",
      "REGISTRATION": "完成账户注册",
      "SUBMITTED": "服务商完成提交资料到⼯信部",
      "ACCEPTED": "⼯商部已受理",
      "NOT_ACCEPTED": "⼯商部不予受理",
      "APPROVED": "⼯商部通过",
      "NOT_APPROVED": "⼯商部未通过",
      "CLOSE": "方案关闭"
    };
  }
  //获取需求列表 
  async rtGetNeedList (pobj) {
    let stdate = new Date(pobj.actionBody.Begin);
    let enddate = new Date(pobj.actionBody.End );
    let query = {where:{}};
    query.where["created_at"] = { [this.db.Op.between]: [stdate, enddate] };
    try {
        let res = await this.needinfoDao.model.findAndCountAll(query);
        res = res.rows.map(n=>n.channelNeedNo);
        res = pobj.actionBody.data.filter(r =>!res.includes(r.bizId));
        return system.getResultSuccess(res)
    } catch (error) {
        return system.getResultFail(-1, error.message)
    }
  }

  //交付商提交材料信息
  async rtSubmitMaterial(pobj) {
    var ab = pobj.actionBody;
    console.log('rt  ab++',ab)
    if (!ab.orderNo) {
      return system.getResultFail(-101, "订单号不能为空");
    }
    var ns = await this.needsolutionDao.model.findOne({
      where: { orderNo: ab.orderNo }, raw: true
    });
    if (!ns || !ns.id) {
      return system.getResultFail(-301, "未知方案");
    }
    ab["bizId"] = ns.channelSolutionNo;
    pobj.actionBody = ab;
    return this.submitRtMaterial(pobj);
  }
  //提交材料信息
  async submitRtMaterial(pobj) {
    var ab = pobj.actionBody;
    console.log('rt ababab++',ab)
    var user = pobj.userInfo;
    if (!user || !user.id) {
      return system.getResultFail(-100, "未知用户");
    }
    if (!ab.bizId) {
      return system.getResultFail(-101, "渠道方案编号不能为空");
    }
    if (!ab.material) {
      return system.getResultFail(-102, "材料信息有误");
    }
    //获取方案信息
    var needsolutioninfo = await this.needsolutionDao.model.findOne({
      attributes: ["id", "needNo", "orderNo", "solutionContent"],
      where: { channelSolutionNo: ab.bizId, isInvalid: 0 }, raw: true
    });
    console.log('needsolutioninfo+++',needsolutioninfo)
    if (!needsolutioninfo || !needsolutioninfo.id) {
      return system.getResultFail(-400, "未知方案");
    }
    if (!needsolutioninfo.needNo) {
      return system.getResultFail(-401, "方案需求信息有误");
    }
    var solutionContent = needsolutioninfo.solutionContent;
    if (!solutionContent) {
      return system.getResultFail(-402, "方案交付信息有误");
    }
    var orderInfo = await this.orderinfoDao.model.findOne({
      where: {orderNo: needsolutioninfo.orderNo}, raw: true
    })
    if(orderInfo){
      var uappId = orderInfo.uapp_id
    }
    
    var material = ab.material;
    if (!material.name) {
      return system.getResultFail(-101, "公司名称不能为空");
    }
    if (!material.partnerBusinessLicense) {
      return system.getResultFail(-103, "营业执照不能为空");
    }
    solutionContent.material = ab.material;
    var solutionFlowList = solutionContent.solutionFlowList || [];
    solutionFlowList.push({
      status: "MATERIAL_UNCONFIRM", statusName: this.rtSolutionStatus.MATERIAL_UNCONFIRM, updated_at: new Date()
    });
    solutionContent.status = "MATERIAL_UNCONFIRM";
    solutionContent.statusName = this.rtSolutionStatus.MATERIAL_UNCONFIRM;
    needsolutioninfo.solutionContent = JSON.stringify(solutionContent);
    return await this.needsolutionDao.db.transaction(async function (t) {
      await this.needsolutionDao.update(needsolutioninfo, t);
      var new_needsolutioninfo = await this.needsolutionDao.model.findOne({
        where: { id: needsolutioninfo.id }, raw: true
      });
      new_needsolutioninfo['orderChannelNo'] = orderInfo.channelOrderNo;
      new_needsolutioninfo['uapp_id'] = uappId;
      console.log('rt  new_needsolutioninfo+++',new_needsolutioninfo)
      return system.getResultSuccess(new_needsolutioninfo);
    })
  }

  //提交方案
  async rtSubmitSolution(pobj) {
    console.log("rt pobj++++",pobj)
    var ab = pobj.actionBody;
    var user = pobj.userInfo;
    if (!user || !user.id) {
      return system.getResultFail(-100, "未知用户");
    }
    ab["createUserId"] = user.id;
    if (!ab.needNo) {
      return system.getResultFail(-101, "渠道需求号不能为空");
    }
    if (!ab.solutionContent) {
      return system.getResultFail(-102, "方案信息有误");
    }
    //获取需求信息
    var needinfo = await this.needinfoDao.model.findOne({
      where: { channelNeedNo: ab.needNo }, raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-201, "未知需求信息");
    }
    if (needinfo.status == "ycd" || needinfo.status == "ygb") {
      return system.getResultFail(-202, "该方案需求状态为" + needinfo.statusName + ",不能创建方案");
    }
    var bizType = needinfo.typeCode;//ab.solutionContent.bizType;//业务类型
    ab.solutionContent.bizType = bizType;
    if (!bizType) {
      return system.getResultFail(-205, "方案类型错误");
    }
    
    if (!needinfo.channelTypeCode) {
      return system.getResultFail(-206, "渠道方案类型错误");
    }
    ab.solutionContent.typeCode = needinfo.typeCode;
    ab.solutionContent.typeName = needinfo.typeName;
    var ns = await this.needsolutionDao.model.findAll({
      where: { channelNeedNo: ab.needNo }, raw: true, order: [["id", 'asc']]
    });
    for (var i = 0; i < ns.length; i++) {
      var fa = ns[i];
      if (fa.status == "dqr" || fa.status == "ywc") {
        return system.getResultFail(-207, "需求方案已存在，不能重复提交");
      }
      if (fa.status == "ybh") {
        ab.channelSolutionNo = fa.channelSolutionNo;
      }
      if (fa.status == "yzf") {
        ab.channelSolutionNo = "";
      }
    }
    var solution = ab.solutionContent.solution;
    if (!solution) {
      return system.getResultFail(-103, "方案交付信息有误");
    }
    if (needinfo.channelTypeCode == "RT") {
      ab.solutionContent.solution.icpType = "RT";
    } else {
      return system.getResultFail(-104, "需求业务类型有误");
    }
    if (!solution.companyName) {
      return system.getResultFail(-105, "公司名不能为空");
    }
    if (!solution.licenseType) {
      return system.getResultFail(-106, "网文类型不能为空");
    }
    if (!solution.area) {
      return system.getResultFail(-107, "区域不能为空");
    }
    if (!solution.actionType) {
      return system.getResultFail(-108, "办理类型不能为空");
    }
    var solutionFlowList = ab.solutionContent.solutionFlowList || [];
    solutionFlowList.push({
      status: "SOLUTION_SUBMIT", statusName: this.rtSolutionStatus.SOLUTION_SUBMIT, updated_at: new Date()
    });
    ab.solutionContent.status = "SOLUTION_SUBMIT";
    ab.solutionContent.statusName = this.rtSolutionStatus.SOLUTION_SUBMIT;
    ab.solutionContent.solutionFlowList = solutionFlowList;
    ab["channelNeedNo"] = needinfo.channelNeedNo;
    var solutionNo = await this.getBusUid("ns");
    ab["solutionNo"] = solutionNo;
    ab["status"] = "dqr";
    var customerInfo = {//客户信息
      "publishName": needinfo.publishName,
      "publishMobile": needinfo.publishMobile
    };
    var salesmanInfo = {//业务员信息
      "salesmanName": user.channel_username,
      "salesmanMobile": user.mobile,
      "salesmanId": user.id,
      "salesmanChannelId": user.channel_userid
    };
    ab.solutionContent["customerInfo"] = customerInfo;
    ab.solutionContent["salesmanInfo"] = salesmanInfo;
    ab.solutionContent = JSON.stringify(ab.solutionContent);
    var self = this;
    return await this.needsolutionDao.db.transaction(async function (t) {
      ab["needNo"] = needinfo.needNo;
      var od = await self.needsolutionDao.create(ab, t);
      if (od && od.id) {
        var needObj = {
          id: needinfo.id
        };
        if (!needinfo.followManUserId) {
          needObj = {
            id: needinfo.id,
            followManUserId: user.id,//跟进人id
            followManName: user.channel_username,//跟进人姓名
            followManMobile: user.mobile,//跟进人手机号（合伙人）
            followManOnlyCode: user.channel_userid
          };
        }
        if (!needinfo.followContent) {
          var followContent = [{
            followDate: new Date(),
            content: "rt提交方案"
          }];
          followContent = JSON.stringify(followContent);
          needObj["followContent"] = followContent;
        }
        await self.needinfoDao.update(needObj, t);
        needinfo = await self.needinfoDao.model.findOne({
          where: { id: needinfo.id }, raw: true
        });
        return system.getResultSuccess({ needinfo: needinfo, needsolution: od });
      } else {
        return system.getResultFail(-302, "提交方案失败");
      }
    })
  }
  
  //交付商通知状态变更
  async rtNotification(pobj) {
    var ab = pobj.actionBody;
    if (!ab.orderNo) {
      return system.getResultFail(-101, "订单号不能为空");
    }
    if (!ab.status) {
      return system.getResultFail(-102, "交付状态不能为空");
    } else {
      ab["applicationStatus"] = ab.status;
    }
    if (ab.officialFileURL) {
      ab["officialFileURL"] = ab.officialFileURL;
    }

    var ns = await this.needsolutionDao.model.findOne({
      where: { orderNo: ab.orderNo }, raw: true
    });
    if (!ns || !ns.id) {
      return system.getResultFail(-301, "未知方案");
    }
    ab["bizId"] = ns.channelSolutionNo;
    ab["solutionNo"] = ns.solutionNo;
    pobj.actionBody = ab;
    return this.rtAcceptPartnerNotification(pobj);
  }
  //服务商通知状态变更
  async rtAcceptPartnerNotification(pobj) {
    console.log('rt 服务商通知状态变更pobj++++',pobj)
    var ab = pobj.actionBody;
    var user = pobj.userInfo;
    
    if (!user || !user.id) {
      return system.getResultFail(-100, "未知用户");
    }
    if (!ab.solutionNo) {
      return system.getResultFail(-101, "方案编号不能为空");
    }
    if (!ab.applicationStatus) {
      return system.getResultFail(-102, "通知状态不能为空");
    }
    //获取方案信息
    var needsolutioninfo = await this.needsolutionDao.model.findOne({
      attributes: ["id", "status", "solutionContent", "needNo"],
      where: { solutionNo: ab.solutionNo }, raw: true
    });
    if (!needsolutioninfo || !needsolutioninfo.id) {
      return system.getResultFail(-400, "未知方案");
    }
    if (!needsolutioninfo.needNo) {
      return system.getResultFail(-401, "方案需求信息有误");
    }
    var solutionContent = needsolutioninfo.solutionContent;
    if (!solutionContent) {
      return system.getResultFail(-402, "方案交付信息有误");
    }
    if (solutionContent.applicationStatus && solutionContent.applicationStatus > ab.applicationStatus) {
      return system.getResultFail(-403, "操作失败，交付流程未按顺序执行");
    }
    if (solutionContent.applicationStatus && solutionContent.applicationStatus == ab.applicationStatus) {
      return system.getResultFail(-405, "操作失败，该流程状态已提交，不能重复提交");
    }
    
    var solutionFlowList = solutionContent.solutionFlowList || [];
    var orderInfo = await this.orderinfoDao.model.findOne({
      attributes:["uapp_id","channelOrderNo"],
      where: {orderNo: ab.orderNo}, raw: true
    })
    console.log("rt orderInfo+++",orderInfo)
    if(orderInfo){
      var uappId = orderInfo.uapp_id;
      var channelOrderNo = orderInfo.channelOrderNo
    }
    if (ab.applicationStatus == 507) {//完成账户注册
      solutionContent.applicationStatus = 507;
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "REGISTRATION", statusName: this.rtSolutionStatus.REGISTRATION, updated_at: new Date()
      });
      solutionContent.status = "REGISTRATION";
      solutionContent.statusName = this.rtSolutionStatus.REGISTRATION;
    }
    if (ab.applicationStatus == 508) {//"服务商完成提交资料到⼯信部
      if (solutionContent.status != "REGISTRATION") {
        return system.getResultFail(-508, "交付流程错误，请先完成账户注册");
      }
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "SUBMITTED", statusName: this.rtSolutionStatus.SUBMITTED, updated_at: new Date()
      });
      solutionContent.status = "SUBMITTED";
      solutionContent.statusName = this.rtSolutionStatus.SUBMITTED;
    }
    if (ab.applicationStatus == 509) {//⼯商部已受理
      if (solutionContent.status != "SUBMITTED") {
        return system.getResultFail(-509, "交付流程错误，请先完成资料递交");
      }
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "ACCEPTED", statusName: this.rtSolutionStatus.ACCEPTED, updated_at: new Date()
      });
      solutionContent.status = "ACCEPTED";
      solutionContent.statusName = this.rtSolutionStatus.ACCEPTED;
    }
    if (ab.applicationStatus == 510) {//⼯商部不予受理
      if (solutionContent.status != "SUBMITTED") {
        return system.getResultFail(-510, "交付流程错误，请先完成资料递交");
      }
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "NOT_ACCEPTED", statusName: this.rtSolutionStatus.NOT_ACCEPTED, updated_at: new Date()
      });
      solutionContent.status = "NOT_ACCEPTED";
      needsolutioninfo.status = "ywc";
      solutionContent.statusName = this.rtSolutionStatus.NOT_ACCEPTED;
    }
    if (ab.applicationStatus == 511) {//⼯商部通过
      if (solutionContent.status != "ACCEPTED") {
        return system.getResultFail(-511, "交付流程错误，⼯信部已受理后才能执行此操作");
      }
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "APPROVED", statusName: this.rtSolutionStatus.APPROVED, updated_at: new Date()
      });
      needsolutioninfo.status = "ywc";
      solutionContent.status = "APPROVED";
      solutionContent.statusName = this.rtSolutionStatus.APPROVED;
    }
    if (ab.applicationStatus == 512) {//⼯信部未通过
      if (solutionContent.status != "ACCEPTED") {
        return system.getResultFail(-512, "交付流程错误，⼯信部已受理后才能执行此操作");
      }
      solutionFlowList.push({
        file: ab.officialFileURL || "",
        status: "NOT_APPROVED", statusName: this.rtSolutionStatus.NOT_APPROVED, updated_at: new Date()
      });
      solutionContent.status = "NOT_APPROVED";
      needsolutioninfo.status = "ywc";
      solutionContent.statusName = this.rtSolutionStatus.NOT_APPROVED;
    }
    solutionContent.solutionFlowList = solutionFlowList;
    var applicationStatusList = solutionContent.applicationStatusList || [];
    var statusObj = {
      "officialFileURL": ab.officialFileURL || "",
      "applicationStatus": ab.applicationStatus,
      "applicationStatusName": this.rtSolutionStatus[ab.applicationStatus],
      "created_at": new Date()
    };
    applicationStatusList.push(statusObj);
    solutionContent.applicationStatusList = applicationStatusList;
    solutionContent.applicationStatus = ab.applicationStatus;
    needsolutioninfo.solutionContent = JSON.stringify(solutionContent);
    
    return await this.needsolutionDao.db.transaction(async function (t) {
      await self.needsolutionDao.update(needsolutioninfo, t);
      statusObj["bizId"] = ab.bizId;
      statusObj['uapp_id'] = uappId;
      statusObj['channelOrderNo'] = channelOrderNo;
      console.log('rt statusObj++',statusObj)
      return system.getResultSuccess(statusObj);
    })
  }

  //接收方案编号(方案推送至yun后,接收保存方案信息)
  async rtReceiveProgrammeNo(pobj) {
    var ab = pobj.actionBody;
    if (!ab.solutionBizId) {
      return system.getResultFail(-101, "渠道方案业务编号不能为空");
    }
    if (!ab.solutionNo) {
      return system.getResultFail(-102, "方案业务编号不能为空");
    }
    var ns = await this.needsolutionDao.model.findOne({
      where: { solutionNo: ab.solutionNo, isInvalid: 0 }, raw: true
    });
    if (!ns || !ns.id) {
      return system.getResultFail(-103, "未知方案");
    }
    var needinfo = await this.needinfoDao.model.findOne({
      where: { needNo: ns.needNo }, raw: true
    });
    if (!needinfo) {
      return system.getResultFail(-104, "未知方案需求");
    }
    if (needinfo.status == "ycd" || needinfo.status == "ygb") {
      return system.getResultFail(-105, "该方案需求状态为" + needinfo.statusName + ",不能执行此操作");
    }
    await this.needsolutionDao.model.update({ channelSolutionNo: ab.solutionBizId }, { where: { id: ns.id } });//修改方案信息
    return system.getResultSuccess();
  }

  /*
  返回20位业务订单号
  prefix：业务前缀
  */
  async getBusUid(prefix) {
    prefix = (prefix || "");
    if (prefix) {
      prefix = prefix.toUpperCase();
    }
    var prefixlength = prefix.length;
    var subLen = 8 - prefixlength;
    var uidStr = "";
    if (subLen > 0) {
      uidStr = await this.getUidInfo(subLen, 60);
    }
    var timStr = moment().format("YYYYMMDDHHmm");
    return prefix + timStr + uidStr;
  }
  /*
  len：返回长度
  radix：参与计算的长度，最大为62
  */
  async getUidInfo(len, radix) {
    var chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');//长度62，到yz长度为长36
    var uuid = [], i;
    radix = radix || chars.length;
    if (len) {
      for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
    } else {
      var r;
      uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
      uuid[14] = '4';
      for (i = 0; i < 36; i++) {
        if (!uuid[i]) {
          r = 0 | Math.random() * 16;
          uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
        }
      }
    }
    return uuid.join('');
  }

  //提交沟通记录
  async rtWriteCommunicationRecord(pobj) {
    // 查询需求沟通记录
    pobj.actionBody.note = ["noteTime", moment().format("YYYY-MM-DD HH:mm:ss"), "note", pobj.actionBody.note];
    let needRes = await this.needsolutionDao.findOne({ channelNeedNo: pobj.actionBody.needNo });
    console.log('needRes',needRes)
    // 没有需求单直接返回
    if (!needRes) {
      return system.getResult("没有这个需求单");
    }
    // 如果未推送/以推送 修改为已跟进
    if (needRes.status == "wts" || needRes.status == "yts") {
      var sql = "update n_need_info set status=:status, statusName=:statusName where channelNeedNo=:bizId"
      var paramWhere = {
        statusName: '已跟进',
        status: 'ygj',
        bizId: pobj.actionBody.needNo
      };
      var updateRes = await this.orderinfoDao.customUpdate(sql, paramWhere);
      console.log('updateRes',updateRes)
    }
    // 有需求单但没有沟通记录 直接set
    if (!needRes.followContent) {
      var sql = "update n_need_info set followContent=JSON_OBJECT(:followContent) where channelNeedNo=:bizId"
      var paramWhere = {
        followContent: pobj.actionBody.note,
        bizId: pobj.actionBody.needNo
      };
      var updateRes = await this.orderinfoDao.customUpdate(sql, paramWhere);
      if (updateRes[1]) {
        return system.getResultSuccess();
      }
      return system.getResult("添加记录失败");
      // 有需求单有沟通记录 json_array_append
    } else {
      var sql = "update n_need_info set followContent=json_array_append(followContent, '$', JSON_OBJECT(:note)) where channelNeedNo=:bizId"
      // var sql = "update n_need_info set followContent=json_array_append(followContent, '$', "+ "\"" + JSON.stringify(pobj.actionBody.note) + "\""+") where channelNeedNo="+ "\"" +pobj.actionBody.bizId +"\"" +"  "
      var paramWhere = {
        note: pobj.actionBody.note,
        bizId: pobj.actionBody.needNo
      };
      var updateRes = await this.orderinfoDao.customUpdate(sql, paramWhere);
      console.log('updateRes',updateRes)
      if (updateRes[1]) {
        return system.getResultSuccess();
      }
      return system.getResult("添加记录失败");
    }
  }

  //查询沟通记录
  async rtGetCommunicationRecord(pobj){
    console.log('query++pobj',pobj)
    if (!pobj.actionBody.intentionBizId) {
      return system.getResult(null, "actionBody.intentionBizId can not be empty，100493");
    }
    if(pobj.appInfo.uapp_id == 18){
      if (!pobj.actionBody.userFeedBack) {
        return system.getResult(null, "actionBody.userFeedBack can not be empty，100494");
      }
    }
    var sql = "select uapp_id from n_need_info where channelNeedNo = :channelNeedNo ";
    var where = {
      channelNeedNo:pobj.actionBody.intentionBizId
    }
    var uappIdInfo = await this.customQuery(sql,where);
    if(uappIdInfo){
      uappIdInfo = uappIdInfo[0];
      return system.getResultSuccess(uappIdInfo);
    }
    
  }

  //关闭需求
  async rtCloseNeed(pobj) {
    var ab = pobj.actionBody;
    var app = pobj.appInfo;
    if (!app || !app.uapp_id) {
      return system.getResultFail(-100, "未知渠道");
    }
    if (!ab.needNo) {
      return system.getResultFail(-101, "需求编号不能为空");
    }
    if (!ab.note) {
      return system.getResultFail(-106, "关闭理由不能为空");
    }
    //获取需求信息
    var needinfo = await this.needinfoDao.model.findOne({
      attributes: ["id", "status", "statusName", "needNo"],
      where: { channelNeedNo: ab.needNo }, raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-201, "未知需求信息");
    }
    if (needinfo.status == "ygb") {
      return system.getResultFail(-202, "该方案需求状态为" + needinfo.statusName + ",不能执行此操作");
    }
    var updateNeed = await this.needinfoDao.update(needObj);//关闭需求
    if (!updateNeed) {
      return system.getResultFail(-203, "关闭需求失败");
    } else {
      return system.getResultSuccess();
    }
  }
  //根据需求关闭方案（关闭需求后调用）
  async rtClosePlan(pobj) {
    var ab = pobj.actionBody;
    var app = pobj.appInfo;
    if (!app || !app.uapp_id) {
      return system.getResultFail(-100, "未知渠道");
    }
    if (!ab.intentionBizId) {
      return system.getResultFail(-101, "需求编号不能为空");
    }
    //获取需求信息
    var needinfo = await this.needinfoDao.model.findOne({
      attributes: ["id", "status", "statusName", "needNo", "uapp_id"],//2020-10-29 laolan xinzeng uapp_id
      where: { channelNeedNo: ab.intentionBizId }, raw: true
    });
    if (!needinfo || !needinfo.id) {
      return system.getResultFail(-201, "未知需求信息");
    }
    if (needinfo.status == "ygb") {
      return system.getResultFail(-202, "该方案需求状态为" + needinfo.statusName + ",不能执行此操作");
    }
    //获取方案信息
    var ns = await this.needsolutionDao.model.findOne({
      where: { needNo: needinfo.needNo, isInvalid: 0 }, raw: true
    });
    if (ns && ns.id) {
      if (ns.status == "ywc") {
        return system.getResultFail(-103, "方案状态错误，不能废弃已完成方案");
      }
      var flowStatus = ns.solutionContent && ns.solutionContent.status ? ns.solutionContent.status : "";
      if (flowStatus && ["NOT_APPROVED", "CLOSE"].indexOf(flowStatus) < 0) {
        return system.getResultFail(-104, "⽅案状态为关闭或不予受理才可关闭需求");
      }
    }
    var self = this;
    return await this.needsolutionDao.db.transaction(async function (t) {
      var needObj = {
        id: needinfo.id, status: "ygb", notes: ab.note
      };
      await self.needinfoDao.update(needObj, t);//关闭需求
      if (ns && ns.id) {
        await self.needsolutionDao.model.update({ status: "yzf", isInvalid: 1 }, { where: { id: ns.id }, transaction: t });//方案废弃
        return system.getResultSuccess(ns.orderNo);
      }
      var uappId = {};
      uappId['uapp_id'] = needinfo.uapp_id;
      ns = ns ? ns : {};
      ns = Object.assign(ns,uappId)
      console.log('rt guanbi ns +++',ns)
      return system.getResultSuccess(ns);
    })

  }
  
  //交付商关闭交付单
  async rtCloseOrderDelivery(pobj) {
    var ab = pobj.actionBody;
    if (!ab.orderNo) {
      return system.getResultFail(-101, "订单号不能为空");
    }
    if (!ab.note) {
      return system.getResultFail(-106, "关闭理由不能为空");
    } else {
      ab.note = ab.note;
    }
    var ns = await this.needsolutionDao.model.findOne({
      where: { orderNo: ab.orderNo }, raw: true
    });

    if (!ns || !ns.id) {
      return system.getResultFail(-301, "未知方案");
    }
    ab["bizId"] = ns.channelSolutionNo;
    pobj.actionBody = ab;
    return this.rtAbolishProgramme(pobj);
  }
  //服务商作废方案-- rtAbolishProgramme
  async rtAbolishProgramme(pobj) {
    var ab = pobj.actionBody;
    var user = pobj.userInfo;
    if (!user || !user.id) {
      return system.getResultFail(-100, "未知用户");
    }
    if (!ab.bizId) {
      return system.getResultFail(-101, "渠道方案号不能为空");
    }
    if (!ab.note) {
      return system.getResultFail(-106, "关闭理由不能为空");
    }
    //获取方案信息
    var ns = await this.needsolutionDao.model.findOne({
      where: { channelSolutionNo: ab.bizId, isInvalid: 0 }, raw: true
    });
    if (!ns || !ns.id) {
      return system.getResultFail(-300, "未知方案");
    }
    var needinfo = await this.needinfoDao.model.findOne({
      where: { needNo: ns.needNo }, raw: true
    });
    if (!needinfo) {
      return system.getResultFail(-400, "未知方案需求");
    }
    if (needinfo.status == "ycd" || needinfo.status == "ygb") {
      return system.getResultFail(-401, "该方案需求状态为" + needinfo.statusName + ",不能执行此操作");
    }
    // if (ns.status != "dqr") {
    //   return system.getResultFail(-301, "方案状态错误，只能废弃待确认方案");
    // }
    var solutionContent = ns.solutionContent;
    if (!solutionContent.status || ["MATERIAL_UNCONFIRM", "REGISTRATION", "SUBMITTED"].indexOf(solutionContent.status) < 0) {
      return system.getResultFail(-302, "方案流程状态错误，不能执行此操作");
    }
    solutionContent.serviceProviderNote = ab.note;
    var solutionFlowList = solutionContent.solutionFlowList || [];
    solutionFlowList.push({
      status: "CLOSE", statusName: this.rtSolutionStatus.CLOSE, updated_at: new Date()
    });
    solutionContent.status = "CLOSE";
    solutionContent.statusName = this.rtSolutionStatus.CLOSE;
    solutionContent = JSON.stringify(solutionContent);
    await this.needsolutionDao.update({ id: ns.id, status: "yzf", isInvalid: 1, solutionContent: solutionContent });//方案废弃
    //获取方案信息
    var new_ns = await this.needsolutionDao.model.findOne({
      where: { id: ns.id }, raw: true
    });
    return system.getResultSuccess(new_ns);
  }
}
module.exports = RtService;
